from logging import Logger

import boto3

from sidecar.const import Const
from sidecar.sandbox_start_time_updater import ISandboxStartTimeUpdater
from sidecar.updater import IStatusUpdater


class AwsStatusUpdater(IStatusUpdater):
    def __init__(self, sandbox_id: str, region_name: str, logger: Logger,
                 sandbox_start_time_updater: ISandboxStartTimeUpdater):
        self.sandbox_start_time_updater = sandbox_start_time_updater
        self.logger = logger
        self.sandbox_id = sandbox_id
        self.region_name = region_name

    def update_status(self, name: str, ip_address: str, status: str):
        self._update_app_status(ip_address, name, status)
        self.sandbox_start_time_updater.update_sandbox_start_time_if_completed(app_name=name, status=status)

    def _update_app_status(self, ip_address: str, app_name: str, status: str):
        instance = self._get_instance(self.region_name, self.sandbox_id, ip_address)
        if instance is None:
            raise Exception("Could not find instance for {}".format(app_name))
        updated_state = self._create_instance_state(instance=instance, status=status, app_name=app_name)
        instance.create_tags(Tags=[self._create_tag(Const.APP_STATUS_TAG, updated_state)])

    @staticmethod
    def _create_tag(key, value):
        return {'Key': key, 'Value': value}

    def _create_instance_state(self, instance, status, app_name) -> str:
        instance_tags = AwsStatusUpdater._get_tags(instance)
        apps_status_value = None
        if Const.APP_STATUS_TAG in instance_tags:
            apps_status_value = instance_tags[Const.APP_STATUS_TAG]
        return self.get_updated_app_status_value(app_name=app_name, app_status=status,
                                                 apps_status_value=apps_status_value)

    @staticmethod
    def get_updated_app_status_value(app_name: str, app_status: str, apps_status_value: str) -> str:
        app_to_status_map = dict()
        if apps_status_value:
            all_statuses = apps_status_value.split(Const.CSV_TAG_VALUE_SEPARATE)
            app_to_status_map = dict(state.split(Const.APP_STATE_KEY_VALUE_SEPARATOR) for state in all_statuses)
        app_to_status_map[app_name] = app_status
        return Const.CSV_TAG_VALUE_SEPARATE.join(["{0}:{1}".format(k, v) for k, v in app_to_status_map.items()])

    @staticmethod
    def _get_instance(region_name: str, sandbox_id: str, private_ip_address: str):
        ec2 = boto3.resource('ec2', region_name=region_name)

        filters = [{'Name': 'tag:' + Const.SANDBOX_ID_TAG,
                    'Values': [sandbox_id]},
                   {'Name': 'private-ip-address',
                    'Values': [private_ip_address]}]

        instance = next(iter(list(ec2.instances.filter(Filters=filters))), None)
        return instance

    @staticmethod
    def _contains_csv_tag_vaAwsSandboxStartTimeUpdaterlue(instance, tag_name, tag_value):
        instance_tags = AwsStatusUpdater._get_tags(instance)

        if tag_name in instance_tags:
            split_tag_values = instance_tags[tag_name].split(Const.CSV_TAG_VALUE_SEPARATE)

            return next((split_tag_value for split_tag_value in split_tag_values
                         if split_tag_value == tag_value), None) is not None
        return False

    @staticmethod
    def _get_tags(resource) -> {}:
        tags = resource.tags
        return dict((tag['Key'], tag['Value']) for tag in tags)
