import json
from logging import Logger

import requests
import urllib3

from sidecar.const import Const
from sidecar.kub_token_provider import KubTokenProvider
from sidecar.sandbox_start_time_updater import ISandboxStartTimeUpdater
from sidecar.updater import IStatusUpdater


class KubernetesStatusUpdater(IStatusUpdater):
    def __init__(self, hostname: str, namespace: str, logger: Logger,
                 sandbox_start_time_updater: ISandboxStartTimeUpdater,
                 kub_token_provider: KubTokenProvider):
        self.kub_token_provider = kub_token_provider
        self.sandbox_start_time_updater = sandbox_start_time_updater
        self.logger = logger
        self.hostname = hostname
        self.namespace = namespace
        self._token = self.kub_token_provider.get_token()

    def update_status(self, name: str, ip_address: str, status: str):
        self.logger.info("start")
        self._disable_secure_warnings()
        self._update_app_status(name, status, ip_address)

    def _update_app_status(self, name: str, status: str, ip_address: str):
        pod_json = self._get_pod_json(ip_address=ip_address)
        apps_info_json = self._get_apps_info_json(pod_json)
        apps_info_json[name]['status'] = status

        s = requests.Session()
        s.headers['Content-Type'] = 'application/strategic-merge-patch+json'
        s.headers['Authorization'] = 'Bearer ' + self._token
        url = '{hostname}/api/v1/namespaces/{namespace}/pods/{pod_name}' \
            .format(hostname=self.hostname,
                    namespace=self.namespace,
                    pod_name=self._get_pod_name(pod_json))
        data_json = {
            'metadata': {
                'annotations': {
                    Const.APPS: json.dumps(apps_info_json)
                }
            }
        }
        metadata_json = json.dumps(data_json)
        res = s.patch(url=url, data=metadata_json, verify=False)
        res.raise_for_status()

        self.sandbox_start_time_updater.update_sandbox_start_time_if_completed(app_name=name, status=status)

    def _disable_secure_warnings(self):
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    def _get_pod_json(self, ip_address: str):
        s = requests.Session()
        s.headers['Content-Type'] = 'application/strategic-merge-patch+json'
        s.headers['Authorization'] = 'Bearer ' + self._token
        url = '{hostname}/api/v1/namespaces/{namespace}/pods'. \
            format(hostname=self.hostname,
                   namespace=self.namespace)
        res = s.get(url=url, verify=False)
        res.raise_for_status()
        pods_json = res.json()
        pods = pods_json['items']
        pod_json = next(iter([pod for pod in pods if pod['status']['podIP'] == ip_address]))
        return pod_json

    @staticmethod
    def _get_pod_name(pod_json) -> str:
        return pod_json['metadata']['name']

    @staticmethod
    def _get_apps_info_json(pod_json) -> {}:
        return json.loads(pod_json['metadata']['annotations'][Const.APPS])


