import json
import logging

from flask import Flask, request
from flask import jsonify

from sidecar.aws_session import AwsSession
from sidecar.aws_status_updater import AwsStatusUpdater
from sidecar.aws_sandbox_start_time_updater import AwsSandboxStartTimeUpdater
from sidecar.const import Const, DateTimeProvider
from sidecar.health_check_monitor import HealthCheckMonitor
from sidecar.kubernetes_status_updater import KubernetesStatusUpdater
from sidecar.kub_sandbox_start_time_updater import KubSandboxStartTimeUpdater
from sidecar.kub_token_provider import KubTokenProvider
from sidecar.utils import Utils

app = Flask(__name__)
logger = None
_health_check_monitor = None


def get_status_updater():
    try:
        with open(Const.get_config_file(), 'r') as conf:
            config_json = json.loads(conf.read())

            provider = config_json["provider"]
            sandbox_id = config_json['sandbox_id']
            apps_json = config_json['apps']
            if provider == "kubernetes":
                kub_token_provider = KubTokenProvider()
                hostname = config_json['kub_api_address']
                sandbox_start_time_updater = KubSandboxStartTimeUpdater(
                    hostname=hostname,
                    namespace=sandbox_id,
                    apps_json=apps_json,
                    date_time_provider=DateTimeProvider(),
                    kub_token_provider=kub_token_provider,
                    logger=logger)
                return KubernetesStatusUpdater(hostname=hostname,
                                               namespace=sandbox_id,
                                               logger=logger,
                                               sandbox_start_time_updater=sandbox_start_time_updater,
                                               kub_token_provider=kub_token_provider)

            if provider == "aws":
                region_name = config_json['region_name']
                aws_session = AwsSession(region_name=region_name)
                sandbox_start_time_updater = AwsSandboxStartTimeUpdater(sandbox_id=sandbox_id, aws_session=aws_session,
                                                                        date_time_provider=DateTimeProvider(),
                                                                        logger=logger,
                                                                        apps_json=apps_json)
                return AwsStatusUpdater(sandbox_id=sandbox_id,
                                        region_name=region_name,
                                        logger=logger,
                                        sandbox_start_time_updater=sandbox_start_time_updater)

            raise Exception('unknown provider {}'.format(provider))
    except Exception as exc:
        logger.exception(exc)
        raise exc


@app.route("/")
def hello():
    logger.info("start")
    return "welcome cs18:sidecar-api"


@app.route("/application/<string:app_name>/health-check", methods=['POST'])
def start_health_check(app_name: str):
    logger.info("start")
    # noinspection PyBroadException
    try:
        ip_address = request.remote_addr
        req = request.json
        script_name = req['script']

        _health_check_monitor.start(ip_address=ip_address, app_name=app_name, script_name=script_name)

        # fail here if validation fails ONLY
        return jsonify(success=True), 202
    except Exception as exc:
        logger.exception(exc)
        return jsonify(success=False, error=str(exc)), 401


@app.route("/application/<string:app_name>/log", methods=['GET'])
def get_application_log(app_name: str):
    logger.info("start")
    # noinspection PyBroadException
    try:
        log = Utils.read_log(app_name=app_name)
        return jsonify(success=True, log=log), 200
    except FileNotFoundError as exc:
        logger.exception(exc)
        return jsonify(success=False, error="log for app {} was not found".format(app_name)), 400
    except Exception as exc:
        logger.exception(exc)
        return jsonify(success=False, error=str(exc)), 401


def run():
    global _health_check_monitor
    global logger
    # logger definitions
    logger = logging.getLogger('sidecar-api')
    hdlr = logging.FileHandler(Const.get_log_file())
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(filename)s %(funcName)s %(message)s')
    hdlr.setFormatter(formatter)
    logger.addHandler(hdlr)
    logger.setLevel(logging.INFO)

    _health_check_monitor = HealthCheckMonitor(updater=get_status_updater(), logger=logger)
    # server startstatus_updater
    app.run(host='0.0.0.0', port=4000)


if __name__ == "__main__":
    run()
